/**
	$Id$

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.examples.extensions.overview;

import java.net.MalformedURLException;
import java.net.URL;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.User;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Severity;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.extensions.NodeValidatorAction;
import net.sf.basedb.util.overview.validator.NameableNodeValidator;

/**
	Validator implementation that check the owner of an item.
	We extend from the {@link NameableNodeValidator} / {@link BasicNodeValidator} 
	class to be able to take advantage of already implemented functionality in the BASE core.
	The rules we check for in this validator are:
	<ul>
	<li>Missing user: {@link #MISSING_OWNER}
	<li>Access denied to user: {@link #DENIED_OWNER}
	<li>Missing URL: {@link #MISSING_URL}
	<li>Invalid URL: {@link #INVALID_URL}
	</ul>
	
	@author Nicklas
	@since 1.5
*/
public class OwnerValidator
	extends NameableNodeValidator<User>
	implements NodeValidatorAction<User>
{

	/**
		Validation rule definition for a missing owner. This can't really happen since
		the core will always set an owner. Default severity level is ERROR.
	*/
	public static Validator MISSING_OWNER = new Validator("missing.owner", "Missing owner",
		"Checks if an owner has been set for an item.", "Missing owner", 
		Severity.ERROR);
	
	/**
		Validation rule definition for access permissions to the owner of an item.
		Default severity level is WARNING.
	*/
	public static Validator DENIED_OWNER = new Validator("denied.owner", "Access denied to owner",
			"Checks if the user is denied access to the owner of an item.", "Access denied to owner", 
			Severity.WARNING);
	
	/**
		Validation rule definition for missing URL for a user. Default severity
		level is WARNING.
	*/
	public static Validator MISSING_URL = new Validator("missing.url", "Missing URL",
			"Checks if an URL has been set for a user.", "Missing URL", 
			Severity.WARNING);
	
	/**
		Validation rule definition for invalid URL for a user. Default severity
		level is ERROR.
	*/
	public static Validator INVALID_URL = new Validator("url.invalid", "Invalid URL",
			"Checks if an URL has a valid syntax.", "Invalid URL",
			Severity.ERROR);
	
	public OwnerValidator()
	{
		// Extending from the BasicNodeValidator provide missing item and permission checks
		super(MISSING_OWNER, DENIED_OWNER);
	}

	/*
		From the NodeValidator interface
		--------------------------------
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node parentNode) 
	{
		super.postValidate(dc, context, node, parentNode);
		
		User owner = (User)node.getItem(dc);
		String url = owner.getUrl();
		
		// Check if the owner has an URL
		if (url == null)
		{
			context.createFailure(MISSING_URL, node, null, new Fix("Specify URL for the user", owner));
		}
		else
		{
			// Check if the URL has a valid syntax
			try
			{
				new URL(url);
			}
			catch (MalformedURLException ex)
			{
				context.createFailure(INVALID_URL, node, "Invalid URL (" + ex.getMessage() + ")", new Fix("Fix the invalid url: " + url, owner));
			}
		}
	}

	/**
		@return Suggested fix is to add a user as the owner of the item
	*/
	@Override
	protected Fix getMissingItemFix(DbControl dc, Node parentNode)
	{
		return new Fix("Set owner of '" + parentNode.getTitle() + "'", parentNode.getItem());
	}
	
}
